import json
from lumaCLI.tests.utils import MANIFEST_JSON, FILE_TXT, INVALID_JSON
from lumaCLI.utils import validate_json, json_to_dict, print_response
from unittest.mock import patch, MagicMock
from io import StringIO
from requests.models import Response


def test_validate_json():
    # Positive test case
    is_json = validate_json(json_path=MANIFEST_JSON)
    assert is_json

    # Positive test case with custom endswith
    is_json = validate_json(json_path=MANIFEST_JSON, endswith=".json")
    assert is_json

    # Negative test case with wrong endswith
    is_json = validate_json(json_path=MANIFEST_JSON, endswith=".txt")
    assert not is_json

    # Negative test case with non-existent file
    is_json = validate_json(json_path="non_existent_file.json")
    assert not is_json

    # Negative test case with a file that is not a .json file
    is_json = validate_json(json_path=FILE_TXT)
    assert not is_json


def test_validate_json_to_dict():
    # Positive test case with valid JSON file
    dict_data = json_to_dict(json_path=MANIFEST_JSON)
    assert isinstance(dict_data, dict)

    # Negative test case with non-existent file
    try:
        dict_data = json_to_dict(json_path="non_existent_file.json")
        assert False, "Expected FileNotFoundError"
    except FileNotFoundError:
        pass

    # Negative test case with a file that is not a .json file
    try:
        dict_data = json_to_dict(json_path=FILE_TXT)
        assert False, "Expected json.decoder.JSONDecodeError"
    except json.decoder.JSONDecodeError:
        pass

    # Negative test case with a JSON file that contains invalid JSON
    try:
        dict_data = json_to_dict(json_path=INVALID_JSON)
        assert False, "Expected json.decoder.JSONDecodeError"
    except json.decoder.JSONDecodeError:
        pass


def test_print_response_success():
    response = MagicMock(spec=Response)
    response.ok = True
    response.json.return_value = {"some": "data"}

    with patch("sys.stdout", new_callable=StringIO) as mock_stdout:
        print_response(response)

        # Check the output
        stdout = mock_stdout.getvalue()
        assert "The dbt ingestion to luma was successful!" in stdout
        assert "Items ingested:" in stdout
        assert str(response.json()) in stdout


def test_print_response_error():
    response = MagicMock(spec=Response)
    response.ok = False
    response.status_code = 400
    response.text = "Bad Request"

    with patch("sys.stdout", new_callable=StringIO) as mock_stdout:
        print_response(response)

        # Check the output
        stdout = mock_stdout.getvalue()
        assert "An HTTP error occurred, response status code" in stdout
        assert str(response.status_code) in stdout
        assert (
            response.text not in stdout
        )  # It seems that 'response.text' is not being printed out, hence the assertion


def test_print_response_error_with_json():
    response = MagicMock(spec=Response)
    response.ok = False
    response.status_code = 400
    response.json.return_value = {"detail": "Bad Request"}

    with patch("sys.stdout", new_callable=StringIO) as mock_stdout:
        print_response(response)

        # Check the output
        stdout = mock_stdout.getvalue()
        assert "An HTTP error occurred, response status code" in stdout
        assert str(response.status_code) in stdout
        assert str(response.json()["detail"]) in stdout


def test_print_response_success_with_json_error():
    response = MagicMock(spec=Response)
    response.ok = True
    response.json.side_effect = Exception("Error at printing items ingested")

    with patch("sys.stdout", new_callable=StringIO) as mock_stdout:
        print_response(response)

        # Check the output
        stdout = mock_stdout.getvalue()
        assert "The dbt ingestion to luma was successful!" in stdout
        assert "Items ingested:" in stdout
        assert "Error at printing items ingested" in stdout



from lumaCLI.utils.dbt_utils import validate_json, json_to_dict, print_response
from lumaCLI.utils.postgres_utils import (
    run_command,
    get_pg_dump_tables_info,
    get_pg_dump_views_info,
    create_conn,
    get_tables_size_info,
    generate_pg_dump_content,
    get_tables_row_counts,
    get_db_metadata,
)



from lumaCLI import app
from typer.testing import CliRunner

runner = CliRunner()


def test_ingest(post_endpoint):
    result = runner.invoke(
        app,
        [
            "postgres",
            "ingest",
            post_endpoint,
        ],
    )
    assert result.exit_code == 0, result.output



from lumaCLI import app
from typer.testing import CliRunner
from lumaCLI.tests.utils import METADATA_DIR


runner = CliRunner()


def test_ingest(post_endpoint):
    result = runner.invoke(
        app,
        [
            "dbt",
            "ingest",
            METADATA_DIR,
            "--endpoint",
            post_endpoint,
        ],
    )

    assert result.exit_code == 0, result.output


def test_send_test_results(post_endpoint):
    result = runner.invoke(
        app,
        [
            "dbt",
            "send-test-results",
            METADATA_DIR,
            "--endpoint",
            post_endpoint,
        ],
    )
    assert result.exit_code == 0, result.output



import json
import os
from pathlib import Path
from rich.panel import Panel
from rich import print


def validate_json(json_path: Path, endswith: str = ".json") -> bool:
    """
    Validates whether the provided file is a valid JSON file and ends with the specified string.

    Args:
        json_path (str): The full path to the file to be validated.
        endswith (str, optional): The string the file should end with. Defaults to ".json".

    Returns:
        bool: True if valid, False otherwise.
    """

    # Check that file exists
    if not json_path.is_file():
        error_message = f"[red]Error[/red]: [yellow]{json_path.absolute()}[/yellow] [blue]is not a file[/blue]"
        print(Panel(error_message))
        return False

    # Check that filename ends with the required string
    if not str(json_path).endswith(endswith):
        error_message = f"[red]Error[/red]: [blue]File[/blue] [yellow]{os.path.basename(json_path.absolute())}[/yellow] [blue]does not have the required structure, it should end with [/blue][yellow]'{endswith}'[/yellow]"
        print(Panel(error_message))
        return False

    return True


def json_to_dict(json_path):
    """
    Converts a JSON file to a dictionary.

    Args:
        json_path (str): The full path to the JSON file.

    Returns:
        dict: The JSON data as a dictionary.
    """
    with open(json_path, "r") as json_file:
        json_data: dict = json.load(json_file)
    return json_data


def print_response(response):
    """
    Prints the HTTP response.

    Args:
        response (Response): The HTTP response to be printed.
    """
    if not response.ok:
        try:
            error_message = f"[red]An HTTP error occurred, response status code[/red]: {response.status_code} {response.json()['detail']}"
        except:
            error_message = f"[red]An HTTP error occurred, response status code[/red]: {response.status_code} {response.text}"
        print(Panel(error_message))
    else:
        success_message = (
            "[green]The dbt ingestion to luma was successful!\nItems ingested:[/green]"
        )
        print(Panel(success_message))
        try:
            print(response.json())
        except:
            print(Panel("[red]Error at printing items ingested[/red]"))



import os
from pathlib import Path
import requests
import typer
from rich import print
from rich.panel import Panel
from lumaCLI.utils import (
    validate_json,
    json_to_dict,
    print_response,
)

app = typer.Typer(no_args_is_help=True, pretty_exceptions_show_locals=False)


def get_current_working_directory(metadata_dir, ctx: typer.Context) -> Path:
    if ctx.resilient_parsing:
        return
    if metadata_dir is not None:
        return metadata_dir
    cwd = Path(os.getcwd())
    print(
        Panel(
            f"[yellow]'metadata_dir' not specified, using current working directory {cwd}[/yellow]"
        )
    )
    return cwd


@app.command()
def ingest(
    metadata_dir: Path = typer.Argument(
        None,
        help="Path to the directory with dbt metadata files. If not provided, current working directory will be used.",
        callback=get_current_working_directory,
        exists=True,
        dir_okay=True,
        resolve_path=True,
    ),
    endpoint: str = typer.Option(
        ...,
        "--endpoint",
        "-e",
        envvar="LUMA_DBT_INGEST_ENDPOINT",
        help="URL of the ingestion endpoint.",
    ),
):
    """
    Sends a bundle of JSON files (manifest.json, catalog.json, sources.json, run_results.json) located in the specified directory to a Luma endpoint.
    If any of these files is not present in the directory, the command will fail. Uses the current working directory if 'metadata_dir' is not specified.
    """

    # Define JSON paths

    manifest_json_path = metadata_dir / "manifest.json"
    catalog_json_path = metadata_dir / "catalog.json"
    sources_json_path = metadata_dir / "sources.json"
    run_results_json_path = metadata_dir / "run_results.json"

    # Validate each JSON file
    is_manifest_json_valid = validate_json(
        json_path=manifest_json_path, endswith="manifest.json"
    )
    is_catalog_json_valid = validate_json(
        json_path=catalog_json_path, endswith="catalog.json"
    )
    is_sources_json_valid = validate_json(
        json_path=sources_json_path, endswith="sources.json"
    )
    is_run_results_json_valid = validate_json(
        json_path=run_results_json_path, endswith="run_results.json"
    )

    if not all(
        [
            is_manifest_json_valid,
            is_catalog_json_valid,
            is_sources_json_valid,
            is_run_results_json_valid,
        ]
    ):
        raise typer.Exit(1)

    # Convert each JSON to dict
    manifest_dict = json_to_dict(json_path=manifest_json_path)
    catalog_dict = json_to_dict(json_path=catalog_json_path)
    sources_dict = json_to_dict(json_path=sources_json_path)
    run_results_dict = json_to_dict(json_path=run_results_json_path)

    # Define bundle dict
    bundle_dict = {
        "manifest_json": manifest_dict,
        "catalog_json": catalog_dict,
        "sources_json": sources_dict,
        "run_results_json": run_results_dict,
    }

    try:
        response = requests.post(
            endpoint, json=bundle_dict, verify=False, timeout=(3.05, 60 * 30)
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_message = (
            "[red]The request has failed. Please check your connection and try again."
        )
        if isinstance(e, requests.exceptions.Timeout):
            error_message += " If you're using a VPN, ensure it's properly connected or try disabling it temporarily."
        elif isinstance(e, requests.exceptions.ConnectionError):
            error_message += (
                " This could be due to maximum retries being exceeded or failure to establish a new connection. "
                "Please check your network configuration."
            )
        print(Panel(error_message + "[/red]"))
        raise typer.Exit(1)

    print_response(response)
    return response


@app.command()
def send_test_results(
    metadata_dir: Path = typer.Argument(
        None,
        help="Path to the directory with the run_results.json file. If not provided, current working directory will be used.",
        callback=get_current_working_directory,
        exists=True,
        dir_okay=True,
        resolve_path=True,
    ),
    endpoint: str = typer.Option(
        ...,
        "--endpoint",
        "-e",
        envvar="LUMA_DBT_SEND-TEST-RESULTS_ENDPOINT",
        help="URL of the ingestion endpoint.",
    ),
):
    """
    Sends the run_results.json file located in the specified directory to a Luma endpoint.
    The command will fail if the run_results.json file is not present in the directory. Uses the current working directory if 'metadata_dir' is not specified.
    """
    if metadata_dir is None:
        metadata_dir = os.getcwd()
        print(
            Panel(
                f"[yellow]'metadata_dir' not specified, using current working directory {metadata_dir}[/yellow]"
            )
        )
    run_results_path = Path(metadata_dir) / "run_results.json"
    is_run_results_json_valid = validate_json(run_results_path, "run_results.json")
    if not is_run_results_json_valid:
        raise typer.Exit(1)

    run_results_dict = json_to_dict(json_path=run_results_path)

    try:
        response = requests.post(
            endpoint, json=run_results_dict, verify=False, timeout=(3.05, 60 * 30)
        )
    except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
        error_message = (
            "[red]The request has failed. Please check your connection and try again."
        )
        if isinstance(e, requests.exceptions.Timeout):
            error_message += " If you're using a VPN, ensure it's properly connected or try disabling it temporarily."
        elif isinstance(e, requests.exceptions.ConnectionError):
            error_message += (
                " This could be due to maximum retries being exceeded or failure to establish a new connection. "
                "Please check your network configuration."
            )
        print(Panel(error_message + "[/red]"))
        raise typer.Exit(1)

    print_response(response)
    return response


if __name__ == "__main__":
    app()



#!/usr/bin/env python

import typer
import lumaCLI.dbt as dbt
import lumaCLI.postgres as postgres
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


app = typer.Typer(
    name="lumaCLI", no_args_is_help=True, pretty_exceptions_show_locals=False
)

app.add_typer(dbt.app, name="dbt")
app.add_typer(postgres.app, name="postgres")


def cli():
    """For python script installation purposes (flit)"""
    app()


if __name__ == "__main__":
    app()



import typer
from typing import List, Dict
import requests
from rich import print
from rich.panel import Panel
from lumaCLI.utils import (
    print_response,
    get_db_metadata,
)


app = typer.Typer(no_args_is_help=True, pretty_exceptions_show_locals=False)


def ask_for_endpoint_if_not_dry_run(endpoint, ctx: typer.Context):
    if ctx.resilient_parsing:
        return
    dry_run = ctx.params.get("dry_run", False)

    if endpoint or dry_run:
        return endpoint

    # Raise a typer.BadParameter exception to simulate the same error as missing 'endpoint'
    raise typer.BadParameter("Missing argument 'ENDPOINT'")


@app.command()
def ingest(
    endpoint: str = typer.Argument(
        None,
        callback=ask_for_endpoint_if_not_dry_run,
        envvar="LUMA_POSTGRES_INGEST_ENDPOINT",
        help="URL of the Luma Postgres ingestion endpoint.",
    ),
    username: str = typer.Option(
        ...,
        "--username",
        "-u",
        envvar="LUMA_POSTGRES_USERNAME",
        help="PostgreSQL username.",
        prompt="PostgreSQL username: ",
    ),
    database: str = typer.Option(
        ...,
        "--database",
        "-d",
        envvar="LUMA_POSTGRES_DATABASE",
        help="PostgreSQL database.",
        prompt="PostgreSQL database: ",
    ),
    host: str = typer.Option(
        "localhost",
        "--host",
        "-h",
        envvar="LUMA_POSTGRES_HOST",
        help="PostgreSQL host.",
    ),
    port: str = typer.Option(
        "5432",
        "--port",
        "-p",
        envvar="LUMA_POSTGRES_PORT",
        help="PostgreSQL port.",
    ),
    password: str = typer.Option(
        ...,
        "--password",
        "-P",
        envvar="LUMA_POSTGRES_PASSWORD",
        help="PostgreSQL password.",
        prompt="PostgreSQL password: ",
        hide_input=True,
    ),
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        "-D",
        help="Whether to do a dry run. Print but not send the payload",
    ),
):
    """
    Creates dictionary with Postgres Metadata Information and sends it to the Luma Postgres ingest endpoint.
    """
    db_metadata: Dict[str, List[Dict]] = get_db_metadata(
        username=username, database=database, host=host, port=port, password=password
    )

    if dry_run:
        print(db_metadata)
        return db_metadata

    try:
        # Send payload to Luma endpoint
        response = requests.post(
            endpoint, json=db_metadata, verify=False, timeout=(3.05, 60 * 30)
        )
        print_response(response)
        return response
    except requests.exceptions.RequestException as e:
        error_message = f"[red]An error occurred while sending the request to the Luma endpoint:[/red] {str(e)}"
        print(Panel(error_message))
        raise typer.Exit(1)


if __name__ == "__main__":
    app()
